/*
*  rcfiles.c
*  RcFiles verze 0.2 cz
*  Knihovna pro prci s rc soubory
*  27.3.2002 Jaroslav Beran berk@volny.cz http://www.volny.cz/berk
*  Licence GNU/GPL2
*  posledni zmena 10.5.2002 - 01
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include "rcfiles.h"


/* GetSections - vraci pri neuspechu NULL
*  pri uspechu ukazatel na pole stringu a pocet stringu v *pocet
*  rcfile je nazev souboru ve kterem hleda sekce
*/
char **GetSections(int *pocet, const char rcfile[]);

/* GetStrs - vraci pri neuspechu NULL
*  pri uspechu ukazatel na pole stringu a pocet stringu v *pocet
*  rcfile je nazev souboru ve kterem hleda sekci
*  sekce je jmeno sekce, ze ktere opise radky (prazdne ignoruje)
*/
char **GetStrs(int *pocet, const char rcfile[], const char sekce[]);

/* GetItem - vraci pri neuspechu NULL
*  pri uspechu ukazatel na string
*  rcfile je nazev souboru ve kterem hleda polozku
*  sekce je jmeno sekce, ze ktere opise polozku
*  polozka je jmeno polozky
*/
char *GetItem(const char rcfile[], const char sekce[], const char polozka[]);

/* IsComment - vraci Ano=1, Ne=0 */
int IsComment(const char s[]);

/* IsEmpty - vraci Ano=1, Ne=0 */
int IsEmpty(const char s[]);

/* IsSection - Ano=1, Ne=0, v sekce vraci jeji jmeno (max 49 znaku) */
int IsSection(const char s[], char sekce[]);

/* StripSp - orizne bile znaky na zacatku a na konci (vc. '\n') */
void StripSp(char s[]);


char **GetSections(int *p_pocet, const char rcfile[])
{
  /* max akceptovana delaka radku je 255 znaku vc '\n' */
  char buff[256], csekce[50], **pp_ret, **pp_pom;
  FILE *fr;
  int n;

  if((fr=fopen(rcfile, "r")) == NULL){
    printf("Nemohu otevrit soubor %s\n", rcfile);
    return(NULL);
  }

  *p_pocet=0;
  pp_ret=NULL;
  while(fgets(buff, 255, fr) !=NULL){
    if(IsComment(buff)) continue;
    if(IsEmpty(buff)) continue;
    if(IsSection(buff, csekce)){;
      (*p_pocet)++;

      /* alokace noveho pole */
      if((pp_pom=(char **) malloc((*p_pocet) * sizeof(char *))) == NULL){
        printf("Neni dostatek pameti\n");
        fclose(fr);
        if(*p_pocet > 1){
          for(n=0; n<(*p_pocet)-1; n++)
            free((void *) *(pp_ret+n));
          free((void *) pp_ret);
        }
        return(NULL);
      }

      /* kopirovani stareho pole do noveho */
      for(n=0; n < (*p_pocet)-1; n++)
        *(pp_pom+n)=*(pp_ret+n);

      /* likvidace stareho pole a validace noveho*/
      if(*p_pocet > 1) free((void *) pp_ret);
      pp_ret=pp_pom;

      /* alokace a naplneni noveho stringu */
      if((*(pp_ret+(*p_pocet)-1)=(char *) malloc((strlen(csekce)+1) * sizeof(char))) == NULL){
        printf("Neni dostatek pameti\n");
        if(*p_pocet > 1){
          for(n=0; n<(*p_pocet)-1; n++)
            free((void *) *(pp_ret+n));
        }
        free((void *) pp_ret);
        fclose(fr);
        return(NULL);
      }
      strcpy(*(pp_ret+(*p_pocet)-1), csekce);
    }
  }

  fclose(fr);
  return(pp_ret);
}


char **GetStrs(int *p_pocet, const char rcfile[], const char sekce[])
{
  /* max akceptovana delaka radku je 255 znaku vc '\n' */
  char buff[256], csekce[50], **pp_ret, **pp_pom;
  FILE *fr;
  int n;

  if((fr=fopen(rcfile, "r")) == NULL){
    printf("Nemohu otevrit soubor %s\n", rcfile);
    return(NULL);
  }

  *p_pocet=0;
  pp_ret=NULL;
  while(fgets(buff, 255, fr) !=NULL){
    if(IsComment(buff)) continue;
    if(IsEmpty(buff)) continue;
    if(IsSection(buff, csekce)) continue;
    if(!strcmp(sekce, csekce)){
      StripSp(buff);
      (*p_pocet)++;

      /* alokace noveho pole */
      if((pp_pom=(char **) malloc((*p_pocet) * sizeof(char *))) == NULL){
        printf("Neni dostatek pameti\n");
        fclose(fr);
        if(*p_pocet > 1){
          for(n=0; n<(*p_pocet)-1; n++)
            free((void *) *(pp_ret+n));
          free((void *) pp_ret);
        }
        return(NULL);
      }

      /* kopirovani stareho pole do noveho */
      for(n=0; n < (*p_pocet)-1; n++)
        *(pp_pom+n)=*(pp_ret+n);

      /* likvidace stareho pole a validace noveho*/
      if(*p_pocet > 1) free((void *) pp_ret);
      pp_ret=pp_pom;

      /* alokace a naplneni noveho stringu */
      if((*(pp_ret+(*p_pocet)-1)=(char *) malloc((strlen(buff)+1) * sizeof(char))) == NULL){
        printf("Neni dostatek pameti\n");
        if(*p_pocet > 1){
          for(n=0; n<(*p_pocet)-1; n++)
            free((void *) *(pp_ret+n));
        }
        free((void *) pp_ret);
        fclose(fr);
        return(NULL);
      }
      strcpy(*(pp_ret+(*p_pocet)-1), buff);
    }
  }

  fclose(fr);
  return(pp_ret);
}


char *GetItem(const char rcfile[], const char sekce[], const char polozka[])
{
  char buff[256], buff_1[256], csekce[50], *p_ret, *p_pom;
  FILE *fr;
  int n, m;

  if((fr=fopen(rcfile, "r")) == NULL){
    printf("Nemohu otevrit soubor %s\n", rcfile);
    return(NULL);
  }

  p_ret=NULL;
  while(fgets(buff, 255, fr) !=NULL){
    if(IsComment(buff)) continue;
    if(IsEmpty(buff)) continue;
    if(IsSection(buff, csekce)) continue;
    if(!strcmp(sekce, csekce)){

      /* nacteni jmena sekce */
      n=0;
      while((buff[n] != '=') && (buff[n] != '\0') && ( n < 255)){
        buff_1[n]=buff[n];
        n++;
      }
      buff_1[n]='\0';

      StripSp(buff_1);
      if(strcmp(polozka, buff_1)) continue;

      /* nacteni hodnoty */
      n++;
      m=0;
      while((buff[n] != '\0') && ( n < 255)){
        buff_1[m]=buff[n];
        n++;
        m++;
      }
      buff_1[m]='\0';
      StripSp(buff_1);

      /* alokace pameti pro string a jeho navrat */
      if ((p_ret=(char *) malloc((strlen(buff_1)+1) * sizeof(char))) == NULL){
        printf("Neni dostatek pameti\n");
        fclose(fr);
        return(NULL);
      }
	  strcpy(p_ret, buff_1);

      /* cteni pokracovani hodnoty na dalsim radku */
      while((p_ret[strlen(p_ret)-1]=='\\') && (fgets(buff, 255, fr) !=NULL)){
		p_ret[strlen(p_ret)-1]='\0';
        if((p_pom=(char *) malloc((strlen(p_ret)+strlen(buff)+1) * sizeof(char)))==NULL){
		  printf("Neni dostatek pameti\n");
		  free((void *) p_ret);
          fclose(fr);
          return(NULL);
		}
		strcpy(p_pom, p_ret);
		strcat(p_pom, buff);
		free((void *) p_ret);
		StripSp(p_pom);
		p_ret=p_pom;
	  }

      fclose(fr);
      return(p_ret);
    }
  }

  fclose(fr);
  return(p_ret);
}


int IsComment(const char s[])
{
  int n=0;

  while(isspace(s[n]))
    n++;

  if(s[n] == '#') return(1);
  return(0);
}


int IsEmpty(const char s[])
{
  int n=0;

  while(isspace(s[n]))
    n++;

  if(s[n] == '\0') return(1);
  return(0);
}


int IsSection(const char s[], char sekce[])
{
  int n=0;
  int m;
  char jmeno[50];

  while(isspace(s[n]))
    n++;

  if(s[n] !='[')
    return(0);
  else 
    n++;

  m=0;
  while((s[n] != ']') && (s[n] != '\0') && (m<49))
    jmeno[m++]=s[n++];
  jmeno[m]='\0';

  if(s[n] !=']') return(0);

  strcpy(sekce, jmeno);
  return(1);
}

void StripSp(char s[])
{
  int prvni,
      delka, 
      n=0;

  while(isspace(s[n]))
    n++;
  prvni=n;

  n=strlen(s)-1;
  while((isspace(s[n]) || (s[n] == '\n')) && (n>prvni))
    n--;
  delka=(n+1)-prvni;

  for(n=0; n<delka; n++)
    s[n]=s[prvni+n];
  s[n]='\0';
}
